#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include <stdio.h>
#include "diskio.h"
#include "pff.h"
#include <stdlib.h>

/* Four-Character Code
 *
 * Folosit pentru a indentifica formate de date
 */
#define FCC(c1, c2, c3, c4) \
	(((DWORD)c4 << 24) + ((DWORD)c3 << 16) + ((WORD)c2 << 8) + (BYTE)c1)

#define NO_WAVS 10
unsigned char Buff1[256];	// Wave output FIFO 
unsigned char Buff2[256];	// Wave output FIFO 
volatile uint8_t read = 1;
volatile uint16_t counter = 0;
volatile uint8_t sync = 0;
FATFS Fs;	// sistemul de fisiere
DIR Dir;	// un director
char wavs[10][30];

uint16_t miliseconds = 0;
uint8_t minutes = 0, seconds = 0, hours = 0;
char time_string[20];

#define NONE 0
#define BTN_PC0 1
#define STOP 0
#define PLAY 1

//controleaza redarea audio
int continue_play = PLAY;
//memoreaza ce buton a fost apasat
int button_pressed = NONE;

unsigned int
randr(unsigned int min, unsigned int max)
{
       double scaled = (double)rand()/RAND_MAX;

       return (max - min +1)*scaled + min;
}

ISR(PCINT2_vect) {
	// TODO BONUS- pentru ambele bonusuri
	// vedeti ce buton este apasat
	// daca butonul apasat este diferit fata de cel anterior
	// opriti redarea audio
	// apoi salvati ce buton a fost apasat

	if(!(PINC & (1<<PC0))) {
		if (button_pressed == NONE) {
			button_pressed = BTN_PC0;
		}
		else {
			button_pressed = NONE;
		}
		continue_play = STOP;
	}
}

ISR(TIMER0_COMPA_vect)
{
	if(sync)
	{
		if(read)
		{
			OCR1A = (unsigned char) Buff1[counter++];

			if(counter == 256)
			{
				read = 0;
				counter = 0;
			}
		} else {
			OCR1A = (unsigned char)Buff2[counter++];

			if(counter == 256)
			{
				read = 1;
				counter = 0;
			}
		} 
	}
}

ISR(TIMER2_COMPA_vect)
{
	miliseconds++;
	if(miliseconds == 1000)
	{
		miliseconds = 0;
		seconds++;

		if(seconds == 60) 
		{
			seconds = 0;
			minutes++;

			if(minutes == 60)
			{
				minutes = 0;
				hours++;

				if(hours == 24) 
					hours = 0;
			}
		}

		sprintf(time_string, "%02d:%02d:%02d", hours, minutes, seconds);
	}
}

void Timer0_init(void)
{
	OCR0A = 0xFF;
	TCCR0A = (1 << WGM01);	// mod CTC top OCR0A
	TCCR0B = (1 << CS01);	// prescaler 8
	TIMSK0 = (1 << OCIE0A);	// intrerupere de output compare A
}

void Timer1_init(void)
{
	OCR0A = 0xFF;
	TCCR1A = (1 << COM1A1) | (1 << COM1A0) | (1 << WGM10);	// mod fastPWM 8-bit; set OC0A on match, clear on bottom
	TCCR1B = (1 << WGM12) | (1 << CS10); 			// prescaler 8
}

void Timer2_init(void)
{
	OCR2A = 0x80;
	TCCR2A = (1 << WGM21);			// mod CTC
	TCCR2B = (1 << CS22) | (1 << CS20);	// prescaler 8
	TIMSK2 = (1 << OCIE2A);			// intrerupere de output compare A
}

void IO_init(void)
{
	DDRD |= (1 << PD5);	
	//DDRB &= ~((1 << PB1) | (1 << PB0));	// PB0, PB1 ca input
	PORTB |= (1 << PB1);	// pull-up-uri pentru PB0 si PB1		// OC1A ca output
	DDRC &= ~(1 << PC0);
	PORTC |= (1 << PC0);
	PCMSK2 |=(1<<PCINT16);
	PCICR |=(1<<PCIE2);
}


/*
 * Incarca header-ul unui fisier
 *
 * @return DWORD - 
 * 	0 => format invalid
 * 	1 => eroare I/O
 * 	>1 => numarul de sample-uri
 
 */
 
//WORD ret;
DWORD load_header(void)
{
	DWORD size;
	WORD ret;

	/* Citeste header-ul (12 octeti) */
	if(pf_read(Buff1, 12, &ret)) 
		return 1;	

	if(ret != 12 || LD_DWORD(Buff1 + 8) != FCC('W','A','V','E')) 
		return 0;

	for(;;)
	{
		/* Citeste Chunk ID si size */
		pf_read(Buff1, 8, &ret);			
		if (ret != 8) 
			return 0;

		size = LD_DWORD(&Buff1[4]);		

		/* Verifica FCC */
		switch (LD_DWORD(&Buff1[0])) 
		{
			/* 'fmt ' chunk */
			case FCC('f','m','t',' '):
				if (size > 100 || size < 16) return 0;			// Verifica size

				/* Citeste continutul */
				pf_read(Buff1, size, &ret);
				if (ret != size || Buff1[0] != 1) return 0;		// Verifica codificarea
				if (Buff1[2] != 1 && Buff1[2] != 2) return 0;		// Verifica canalele	
				if (Buff1[14] != 8 && Buff1[14] != 16) return 0;	// Verifica resolutia

				/* Seteaza frecventa */
				OCR0A = (BYTE)(F_CPU / 8 / LD_WORD(&Buff1[4])) - 1;
				break;

			/* 'data' chunk => incepe redarea */
			case FCC('d','a','t','a') :				
				return size;

			/* 'LIST' chunk => skip */
			case FCC('L','I','S','T') :				
			/* 'fact' chunk => skip */
			case FCC('f','a','c','t') :				
				pf_lseek(Fs.fptr + size);
				break;

			/* Chunk necunoscut => eroare */
			default :								
				return 0;
		}
	}

	return 0;
}

/*
 * Functie care reda un fisier audio
 *
 * path - calea absoluta a fisierului
 *
 * @return 0 daca a rulat cu succes fisierul
 */
UINT play (const char *path)
{
	WORD ret;
	FRESULT res;
	unsigned char read2;
	uint16_t buf_size = 256;
	
	DWORD current_size;
	

	if((res = pf_open(path)) == FR_OK) 
	{

		Timer0_init();
		Timer1_init();

		/* Incarca header-ul fisierului */
		current_size = load_header();			
		
		if(current_size < 256) 
			return (UINT)current_size;

		/* Snip sector unaligned part */
		pf_read(Buff1, 512 - (Fs.fptr % 512), &ret);	
		//current_size -= ret;

		res = pf_read(Buff1, buf_size, &ret);
		if(res != FR_OK) 
			return res; 
		
		sync = 1;
		do 
		{		
			read2 = read;

			if(read == 1)	
				res = pf_read(Buff2, buf_size, &ret);
			else
				res = pf_read(Buff1, buf_size, &ret);

			if(res != FR_OK) 
				break; 

			while(read2 == read);

			if(!(PINB & (1 << PB1))) 
			{
				sync = 0;
				_delay_ms(500); 
				break;
			}


		} while(continue_play == PLAY && ret == buf_size);	// Repeta pana cand toate datele au fost citite

		sync = 0;
		res = 0;
	}

	return res;
}

/*
 * TODO 1- Listeaza continutul directorului curent
 */
void list(const char *dir)
{
   //  trebuie sa deschideti folderul curent si sa verificati daca a fost deschis cu succes
   // apoi cititi toate fisierele din folder cu functia pf_readdir din pff.c
   // aveti grija ca in cazul in care un fisier nu exista/nu are nume, vectorul char cu nume nu este vid, este plin cu 0
	if(pf_opendir(&Dir, dir)) {
		return;
	} 
	
	FILINFO info;
	/* Listeaza continutul directorului */
	while (!pf_readdir(&Dir, &info) )
	{
		
		if(info.fname != NULL && info.fname[0]) {
			  // Afiseaza numele fisierului
			_delay_ms(200);	
		}
	}
	  
}

/* TODO 2 - citeste primele 2 fisiere si memoreaza-le pentru a putea fi folosite ulterior */
void getFirstTwoWAv(const char *dir) {


	FILINFO info;
	int file_found = 0;
	
   // trebuie sa deschideti folderul curent si sa verificati daca a fost deschis cu succes
   // apoi cititi toate fisierele din folder cu functia pf_readdir din pff.c 
   // aveti grija ca in cazul in care un fisier nu exista/nu are nume, vectorul char cu nume nu este vid, este plin cu 0
   // in momentul in care un fisier contine subsirul ".WAV" si nu este director
   // puteti sa vedeti ce macrouri aveti in pff.h
   
   //NOTA - daca nu implementati bonusul 1, atunci nu aveti nevoie sa memorati decat primul fisier WAV
   
	if(pf_opendir(&Dir, dir)) {
					return;
	} 

	while (!pf_readdir(&Dir, &info) && file_found < NO_WAVS)
	{	
		if(info.fname != NULL && info.fname[0] && strstr(info.fname, ".WAV")) {
			
			if (!(info.fattrib & (AM_DIR|AM_HID)) ) {
				sprintf(wavs[file_found], "music/%s", info.fname);
				
				file_found++;
			}
		}
	}
}

/*TODO 2 - redati primul fisier din folder*/
void task2(){
	char dir[10] = {"music"};
	
	getFirstTwoWAv(dir);
	PORTD ^= (1 << PD7);
	_delay_ms(500);
	// bonus 1 , in functie de ce buton este apasat, redati una dintre cele 2 melodii salvate
	if(button_pressed == BTN_PC0) {
		play(wavs[randr(0, NO_WAVS - 1)]);
		
	} else {
		play(wavs[randr(0, NO_WAVS - 1)]);
	}
	
}
int main (void) {	
	IO_init();
	Timer2_init();
	sei();
	/* Monteaza sistemul de fisiere */
	pf_mount(&Fs);			
	DDRD |= (1 << PD7);

	button_pressed = NONE;
	for(;;)
	{
		_delay_ms(1000);	
		PORTD ^= (1 << PD7);						/* Delay 1000ms for things to settle down */
		
		if (pf_mount(&Fs)) 
		{	
			continue;	/* Initialize FS */
		}
		

		/* Incearca sa citesti directorul radacina */
		
		while(1) {
			continue_play = PLAY;
			//play("borgore.wav");
			//task1();
			task2();
			//task4();
		}
	
	}

	return 0;
}
